import yaml
from datetime import datetime

from src.data.copyData import copy_images_with_exclusion
from src.data.modelData import get_test_data, get_train_data, define_augmentation_rules
from src.enum.activierungsfunktionEnum import Aktivierungsfunktion
from src.enum.marktEnum import Markt
from src.enum.optimierungsverfahren import Optimierungsverfahren
from src.knn.configKNN import ConfigKNN
from src.knn.createKNN import evaluate_model, fit_model
from src.knn.defineKNN import define_model
from src.result.createExcelFile import create_excel_result
from src.result.customCallback import CustomCallback
from src.result.plotResult import plot_values


# run the test harness for evaluating a model
def run_model():
    print("Lese Configurationsdatei")
    # Lesen der Configurationdatei
    with open("ressources/config/config.yaml", "r") as file:
        config = yaml.load(file, Loader=yaml.FullLoader)
    # Initialisieren der KNN-Modellbau Eigenschaften
    config_knn = ConfigKNN(
        excluded_folder=Markt.Kein_Markt,
        activation_function_1_units=Aktivierungsfunktion.sigmoid,
        activation_function_128_units=Aktivierungsfunktion.ReLU,
        optimization_method=Optimierungsverfahren.SGD
    )

    print("Initialisiere Callback")
    # Erstellen Sie eine Instanz des benutzerdefinierten Callbacks
    callback = CustomCallback()

    print("Generiere Ordner mit OOS und !OOS Daten")
    # Erstellen der Ordner mit Inhalt für die Trainings-/Testdaten und Evaluationsdaten, die für das nächste Modell
    # benötigt werden
    copy_images_with_exclusion(config["bilder"]["original_path"],
                               config["bilder"]["knn_path"],
                               config_knn.excluded_folder)

    print("Starte Zeit")
    start_time = datetime.now()
    # define model
    print("Definiere KNN Modell")
    model = define_model(config_knn)
    # create data generator
    print("Initialisiere ImageDataGenerator")
    datagen = define_augmentation_rules()
    # prepare iterator
    print("Definiere Trainings- und Testdaten Set")
    train_it = get_train_data(datagen, config["bilder"]["knn_path"])
    test_it = get_test_data(datagen, config["bilder"]["knn_path"])
    print("Trainiere das Modell")
    # fit model
    history = fit_model(config, model, train_it, test_it, callback)
    # evaluate model
    print("Evaluiere das Modell")
    evaluate_model(model, test_it)
    # print duration
    print("Stoppe die Zeit")
    end_time = datetime.now()
    print('Die Dauer für das Erstellen des Modells beträgt: {}'.format(end_time - start_time))
    # learning curves
    print("Erstelle den Plot Graphen")
    plot_values(history, config)
    print("Erstelle die Excel-Tabelle")
    create_excel_result(callback, config)


# entry point, run the test harness
run_model()
